/*
    Block class for BlockGame. 
    Has knowledge of grid position, colour and distance to drop
    down grid. 
    Copyright (C) 2003 Irene Cummings (i.n.c@ezrs.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package inc.games;

import inc.games.*;
import inc.Comparable;

public class Block implements Comparable
{
	public int x=0;
	public int y=0;
	public int colour=0;
	private int distance=0;

	public Block()
	{
		//System.out.println("Block()");
	}

	public Block(int i, int j, int rbg)
	{
		//String debug="Block("+i+", "+j+", "+rbg+")";
		//System.out.println(debug);
		init(i,j,rbg);
	}

	protected void init(int i, int j, int rbg)
	{
		x=i;
		y=j;
		colour=rbg;
	}

	//Find all my neighbours of the same colour 
	//
	//This should never be called on blocks already
	//stored in colour block
	//
	public void findNeighbours(ColourGroup cGroup, BlockTable blockT)
	{
		//System.out.println("find nbrs to block @("+x+","+y+")");
		cGroup.AddElement(this);

		//now consider my neighbours
		this.findNeighbours(blockT.GetBlockNearby(this,0,1),cGroup,blockT);
		this.findNeighbours(blockT.GetBlockNearby(this,0,-1),cGroup,blockT);
		this.findNeighbours(blockT.GetBlockNearby(this,1,0),cGroup,blockT);
		this.findNeighbours(blockT.GetBlockNearby(this,-1,0),cGroup,blockT);
	}

	private void findNeighbours(Block nbr, ColourGroup cGroup, BlockTable blockT)
	{
		if (nbr!=null && nbr.colour==colour)
		{
			boolean done=cGroup.IsPresent(nbr);
			if (false==done)
			{
				nbr.findNeighbours(cGroup,blockT);
			}
		}
	}

	public void IncreaseDrop(int dist)
	{
		distance+=dist;
	}

	public boolean UpDate()
	{
		boolean updated=false;
		if (distance>0)
		{
			distance--;
			y++;
			updated=true;
			//System.out.println(this.Print()+" updated");
		}
		return updated;
	}

	public Block Copy()
	{
		Block cpy=new Block(this.x,this.y,this.colour);
		cpy.distance=this.distance;
		return cpy;
	}

	//-1 rhs greater, 0 same, 1 this greater
	public int compare(Comparable rhs)
	{
		int result=-2;
		int rhsX=((Block)rhs).x;
		int rhsY=((Block)rhs).y;
		if (this.x>rhsX)
		{
			result=1;
		}
		else if (this.x<rhsX)
		{
			result=-1;
		}
		else if (this.y>rhsY)
		{
			result=1;
		}
		else if (this.y<rhsY)
		{
			result=-1;
		}
		else
		{
			result=0;
		}
		return result;
	}

	//debug function
	public String Print()
	{
		return new String("(x,y,colour,distance)=("+x+","+y+","+colour+","+distance+")");
	}
}





