/*
    HighScoreBase for J2SE/J2ME
    Copyright (C) 2004 Irene Cummings (i.n.c@ezrs.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package inc.games;

import java.util.*;
import java.io.*;

abstract public class HighScoreBase
{
	public HighScoreBase() {};
	protected static final int m_RecordLength=24;	//flat files need set sizes

	//Record creation, translation & comparison

	protected byte[] CreateRecord(int score, final String nickName, long date)
	{
		//System.out.println("CreateRecord("+score+","+nickName+","+date+")");
		ByteArrayOutputStream byteStream=new ByteArrayOutputStream();
		DataOutputStream outputStream=new DataOutputStream(byteStream);
		try
		{
			outputStream.writeInt(score);		//4 bytes, high byte first
			outputStream.writeLong(date);		//8 bytes
			//allow 12 bytes (6 chars) for our name
			int size=nickName.length();
			if (size>=6)
			{
				outputStream.writeChars(nickName.substring(1,6));
			}
			else
			{
				String newStr=new String(nickName);
				for (int i=1;i<=6-size;i++)
				{
					newStr+=" ";
				}
				outputStream.writeChars(newStr);
			}
			outputStream.flush();
		}
		catch (IOException e)
		{
			//System.out.println("CreateRecord("+score+","+nickName+","+date+") failed");
		}
		return byteStream.toByteArray();
	}

	protected void DecomposeRecord(final byte[] record, int[] score, StringBuffer nickName, long[] date)
	{
		//System.out.println("DecomposeRecord()");
		ByteArrayInputStream byteStream=new ByteArrayInputStream(record);
		DataInputStream inputStream=new DataInputStream(byteStream);
		try
		{
			score[0]=inputStream.readInt();
			date[0]=inputStream.readLong();
			byte[] nameB=new byte[12];
			inputStream.read(nameB);
			String name=new String(nameB);
			nickName.append(name.trim());
			//System.out.println("DecomposeRecord found "+score+","+nickName+","+date);
		}
		catch (IOException e)
		{
			//System.out.println("DecomposeRecord failed");
		}
	}

	protected StringBuffer LongDateToString(long dateL)
	{
		StringBuffer date=new StringBuffer();
		Date dateD=new Date(dateL);
		Calendar cal=Calendar.getInstance();	//hope this is diff each time!
		cal.setTime(dateD);			//hence this not changing system!
		int year=cal.get(Calendar.YEAR);
		year-=2000;	//good for the nxt 1000 yrs!
		int month=cal.get(Calendar.MONTH);
		int day=cal.get(Calendar.DATE);
		int hour=cal.get(Calendar.HOUR);
		int min=cal.get(Calendar.MINUTE);
		date.append(hour);
		date.append(":");
		date.append(min);
		date.append(" ");
		date.append(day);
		date.append("/");
		date.append(month);
		date.append("/");
		date.append(year);
		return date;
	}

	//returns wether usr has made the hs table
	//-1 too low, 0 equal, 1 good enough to add, 2 table not full (-2 error)
	abstract public int IsScoreGood(int score);
}









